@@ -5,7 +5,7 @@ use crate::{
         DEFAULT_CONTENT_ENCODING_CHECKS_AND_CONVERTERS,
     },
     content_media_type::{ContentMediaTypeCheckType, DEFAULT_CONTENT_MEDIA_TYPE_CHECKS},
-    resolver::Resolver,
+    resolver::{DefaultResolver, Resolver, SchemaResolver},
     schemas, ValidationError,
 };
 use ahash::AHashMap;
@@ -212,6 +212,7 @@ lazy_static::lazy_static! {
 /// content media types and more (check the exposed methods)
 #[derive(Clone)]
 pub struct CompilationOptions {
+    external_resolver: Arc<dyn SchemaResolver>,
     draft: Option<schemas::Draft>,
     content_media_type_checks: AHashMap<&'static str, Option<ContentMediaTypeCheckType>>,
     content_encoding_checks_and_converters:
@@ -225,6 +226,7 @@ pub struct CompilationOptions {
 impl Default for CompilationOptions {
     fn default() -> Self {
         CompilationOptions {
+            external_resolver: Arc::new(DefaultResolver),
             validate_schema: true,
             draft: Option::default(),
             content_media_type_checks: AHashMap::default(),
@@ -263,6 +265,8 @@ impl CompilationOptions {
                 config.with_draft(draft);
             }
         }
+        let config = Arc::new(config);
+
         let draft = config.draft();
 
         let scope = match schemas::id_of(draft, schema) {
@@ -270,8 +274,14 @@ impl CompilationOptions {
             None => DEFAULT_SCOPE.clone(),
         };
         let schema_json = Arc::new(schema.clone());
-        let resolver = Resolver::new(draft, &scope, schema_json.clone(), self.store.clone())?;
-        let context = CompilationContext::new(scope.into(), &config);
+        let resolver = Arc::new(Resolver::new(
+            self.external_resolver.clone(),
+            draft,
+            &scope,
+            schema_json,
+            self.store.clone(),
+        )?);
+        let context = CompilationContext::new(scope.into(), Arc::clone(&config), resolver);
 
         if self.validate_schema {
             if let Some(mut errors) = META_SCHEMA_VALIDATORS
@@ -286,12 +296,7 @@ impl CompilationOptions {
 
         let node = compile_validators(schema, &context)?;
 
-        Ok(JSONSchema {
-            schema: schema_json,
-            node,
-            resolver,
-            config,
-        })
+        Ok(JSONSchema { node, config })
     }
 
     /// Ensure that the schema is going to be compiled using the defined Draft.
@@ -347,6 +352,12 @@ impl CompilationOptions {
         self
     }
 
+    /// Use a custom resolver for resolving external schema references.
+    pub fn with_resolver(&mut self, resolver: impl SchemaResolver + 'static) -> &mut Self {
+        self.external_resolver = Arc::new(resolver);
+        self
+    }
+
     /// Ensure that compiled schema is not supporting the provided content media type.
     ///
     /// ```rust
