@@ -1,7 +1,6 @@
 use crate::paths::{InstancePath, JSONPointer};
 
 use crate::{
-    compilation::JSONSchema,
     error::{error, ErrorIterator, ValidationError},
     keywords::CompilationResult,
     validator::Validate,
@@ -18,16 +17,15 @@ impl FalseValidator {
     }
 }
 impl Validate for FalseValidator {
-    fn is_valid(&self, _: &JSONSchema, _: &Value) -> bool {
+    fn is_valid(&self, _: &Value) -> bool {
         false
     }
 
-    fn validate<'a, 'b>(
+    fn validate<'instance>(
         &self,
-        _: &'a JSONSchema,
-        instance: &'b Value,
+        instance: &'instance Value,
         instance_path: &InstancePath,
-    ) -> ErrorIterator<'b> {
+    ) -> ErrorIterator<'instance> {
         error(ValidationError::false_schema(
             self.schema_path.clone(),
             instance_path.into(),
