@@ -1,6 +1,11 @@
 use num_cmp::NumCmp;
 use serde_json::{Map, Value};
 
+use crate::{
+    compilation::context::CompilationContext, paths::JSONPointer, primitive_type::PrimitiveType,
+    ValidationError,
+};
+
 macro_rules! num_cmp {
     ($left:expr, $right:expr) => {
         if let Some(b) = $right.as_u64() {
@@ -37,7 +42,16 @@ pub(crate) fn equal(left: &Value, right: &Value) -> bool {
 
 #[inline]
 pub(crate) fn equal_arrays(left: &[Value], right: &[Value]) -> bool {
-    left.len() == right.len() && left.iter().zip(right.iter()).all(|(a, b)| equal(a, b))
+    left.len() == right.len() && {
+        let mut idx = 0_usize;
+        while idx < left.len() {
+            if !equal(&left[idx], &right[idx]) {
+                return false;
+            }
+            idx += 1;
+        }
+        true
+    }
 }
 
 #[inline]
@@ -49,6 +63,47 @@ pub(crate) fn equal_objects(left: &Map<String, Value>, right: &Map<String, Value
             .all(|((ka, va), (kb, vb))| ka == kb && equal(va, vb))
 }
 
+#[inline]
+pub(crate) fn map_get_u64<'a>(
+    m: &'a Map<String, Value>,
+    context: &CompilationContext,
+    type_name: &str,
+) -> Option<Result<u64, ValidationError<'a>>> {
+    let value = m.get(type_name)?;
+    match value.as_u64() {
+        Some(n) => Some(Ok(n)),
+        None if value.is_i64() => Some(Err(ValidationError::minimum(
+            JSONPointer::default(),
+            context.clone().into_pointer(),
+            value,
+            0.into(),
+        ))),
+        None => Some(Err(ValidationError::single_type_error(
+            JSONPointer::default(),
+            context.clone().into_pointer(),
+            value,
+            PrimitiveType::Integer,
+        ))),
+    }
+}
+
+/// Fail if the input value is not `u64`.
+pub(crate) fn fail_on_non_positive_integer(
+    value: &Value,
+    instance_path: JSONPointer,
+) -> ValidationError<'_> {
+    if value.is_i64() {
+        ValidationError::minimum(JSONPointer::default(), instance_path, value, 0.into())
+    } else {
+        ValidationError::single_type_error(
+            JSONPointer::default(),
+            instance_path,
+            value,
+            PrimitiveType::Integer,
+        )
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::equal;
