@@ -105,7 +105,7 @@ impl<'a, 'b> Output<'a, 'b> {
     /// ```
     pub fn basic(&self) -> BasicOutput<'a> {
         self.root_node
-            .apply_rooted(self.schema, self.instance, &InstancePath::new())
+            .apply_rooted(self.instance, &InstancePath::new())
     }
 }
 
@@ -120,7 +120,8 @@ pub enum BasicOutput<'a> {
 }
 
 impl<'a> BasicOutput<'a> {
-    pub(crate) const fn is_valid(&self) -> bool {
+    /// A shortcut to check whether the output represents passed validation.
+    pub const fn is_valid(&self) -> bool {
         match self {
             BasicOutput::Valid(..) => true,
             BasicOutput::Invalid(..) => false,
@@ -313,7 +314,7 @@ pub struct ErrorDescription(String);
 
 impl fmt::Display for ErrorDescription {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "{}", self.0)
+        f.write_str(&self.0)
     }
 }
 
